/*  smplayer, GUI front-end for mplayer.
    Copyright (C) 2007 Ricardo Villalba <rvm@escomposlinux.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "gui.h"

#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qstatusbar.h>
#include <qlabel.h>
#include <qslider.h>
#include <qpopupmenu.h>
#include <qfileinfo.h>
#include <qtextbrowser.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qapplication.h>
#include <qinputdialog.h>
#include <qcursor.h>
#include <qmenubar.h>
#include <qtimer.h>
#include <qstyle.h>
#include <qdockwindow.h>
#include <qdragobject.h>
#include <qstrlist.h>
#include <qurl.h>
#include <qregexp.h>
#include <cmath>

#include "mplayerwindow.h"
#include "desktopinfo.h"
#include "constants.h"
#include "version.h"
#include "helper.h"
#include "global.h"
#include "translator.h"
#include "keys.h"
#include "images2.h"
#include "preferences.h"
#include "inforeader.h"
#include "timeslider.h"
#include "preferencesdialog.h"
#include "logwindow.h"
#include "playlist.h"
#include "filepreferencesdialog.h"
#include "eqslider.h"
#include "videoequalizer.h"
#include "infowindow.h"
#include "inputdvddirectory.h"
#include "recents.h"

#include "client.h"
#include "server.h"

Gui::Gui( QString file, QWidget* parent, const char* name, WFlags fl ) 
	: GuiBase( parent, name, fl ) 
{
	initial_file = file;
    last_second = 0;

	// Create objects:
	kl = key_list;
	recents = new Recents(this, "recents list");

    mplayerwindow = new MplayerWindow( panel );
    panelLayout = new QGridLayout( panel, 1, 1, 0, 0 );
    panelLayout->addWidget( mplayerwindow, 0, 0 );

	core = new Core( mplayerwindow, this, "core" );

    mplayer_log_window = new LogWindow(this, "mplayer_log_window");
	smplayer_log_window = new LogWindow(this, "smplayer_log_window");

	info_window = new InfoWindow(0,"info_window");

#if DOCK_PLAYLIST
	playlistdock = new QDockWindow(QDockWindow::OutsideDock, this, "playlist" );
	playlist = new Playlist(core, playlistdock);

	playlistdock->setWidget(playlist);
	playlistdock->setResizeEnabled(TRUE);
	playlistdock->setCloseMode(QDockWindow::Always);
	playlistdock->setVerticallyStretchable(TRUE);
	addDockWindow(playlistdock, Qt::DockBottom );
	//setDockEnabled(Qt::DockBottom, TRUE);
	playlistdock->hide();
#else
	playlist = new Playlist(core, 0);
	//playlist = new Playlist(core, this, "playlist", Qt::WType_TopLevel );
	//playlist->show();
#endif

	// Equalizer
	equalizer = new VideoEqualizer(this, "video_equalizer", 
                                   Qt::WType_Dialog );

	// Fullscreen widget
	#if !FULLSCREEN_WITH_MAINWINDOW
	fullscreen_widget = new QWidget(this, "fullscreen_widget",
                                    Qt::WType_TopLevel |
                                    Qt::WStyle_Customize |
                                    Qt::WStyle_NoBorder |
                                    Qt::WStyle_StaysOnTop );
	fullscreen_widget_layout = new QGridLayout( fullscreen_widget, 1, 1, 0, 0 );
	#endif

	// Connects:
	connect( core, SIGNAL(menusNeedInitialize()),
             this, SLOT(initializeMenus()) );
	connect( core, SIGNAL(widgetsNeedUpdate()),
             this, SLOT(updateWidgets()) );
	connect( core, SIGNAL(equalizerNeedsUpdate()),
             this, SLOT(updateEqualizer()) );
	connect( core, SIGNAL(showTime(double)), 
             this, SLOT(displayTime(double)) );
	connect( core, SIGNAL(showFrame(int)),
             this, SLOT(displayFrame(int)) );
	connect( core, SIGNAL(needResize(int, int)),
             this, SLOT(resizeWindow(int,int)) );
	connect( core, SIGNAL(showMessage(QString)),
             this, SLOT(displayMessage(QString)) );
	connect( core, SIGNAL(stateChanged(int)),
             this, SLOT(displayState(int)) );

	connect( core, SIGNAL(mediaStartPlay()),
             this, SLOT(enterFullscreen()) );
	connect( core, SIGNAL(mediaStoppedByUser()),
             this, SLOT(exitFullscreen()) );
	connect( core, SIGNAL(mediaLoaded()),
             this, SLOT(updateMediaInfo()) );
	connect( playlist, SIGNAL(playlistEnded()),
             this, SLOT(exitFullscreen()) );
	connect( playlist, SIGNAL(visibilityChanged()),
             this, SLOT(updatePlaylistButton()) );

	// Hide mplayer window
	connect( core, SIGNAL(noVideo()),
             this, SLOT(hidePanel()) );

	// mplayerwindow
    connect( mplayerwindow, SIGNAL(rightButtonReleased(QPoint)),
	         this, SLOT(showPopupMenu(QPoint)) );
	
	connect( mplayerwindow, SIGNAL(keyPressed(QKeyEvent*)),
			 this, SLOT(keyPressEvent(QKeyEvent *)) );

	// mplayerwindow mouse events
	/*
	connect( mplayerwindow, SIGNAL(doubleClicked()),
             this, SLOT(toggleFullscreen()) );
	*/
	connect( mplayerwindow, SIGNAL(doubleClicked()),
             this, SLOT(doubleClickFunction()) );
	connect( mplayerwindow, SIGNAL(leftClicked()),
             this, SLOT(leftClickFunction()) );
	/*
	connect( this, SIGNAL(doubleClicked()),
             this, SLOT(doubleClickFunction()) );
	connect( this, SIGNAL(leftClicked()),
             this, SLOT(leftClickFunction()) );
	*/
	#if FULLSCREEN_WITH_MAINWINDOW
	connect( mplayerwindow, SIGNAL(mouseMoved(QPoint)),
             this, SLOT(checkMousePos(QPoint)) );
	#endif

	// Mouse Wheel
	connect( this, SIGNAL(wheelUp()),
             core, SLOT(wheelUp()) );
	connect( this, SIGNAL(wheelDown()),
             core, SLOT(wheelDown()) );
	connect( mplayerwindow, SIGNAL(wheelUp()),
             core, SLOT(wheelUp()) );
	connect( mplayerwindow, SIGNAL(wheelDown()),
             core, SLOT(wheelDown()) );


	// Set style before changing color of widgets:
	// Set style
#if STYLE_SWITCHING
	#if QT_VERSION >= 0x040000
	qDebug( "Style name: '%s'", qApp->style()->name() );
	qDebug( "Style class name: '%s'", qApp->style()->className() );
	default_style = qApp->style()->name();
	#else
	qDebug( "Style name: '%s'", qApp->style().name() );
	qDebug( "Style class name: '%s'", qApp->style().className() );
	default_style = qApp->style().name();
	#endif
	if (!pref->style.isEmpty()) {
		qApp->setStyle( pref->style );
	}
#endif

	// Colors:

	// statusbar
	time_display = new QLabel( statusBar() );
	time_display->setAlignment(Qt::AlignRight);

	frame_display = new QLabel( statusBar() );
	frame_display->setAlignment(Qt::AlignRight);
    #if QT_VERSION >= 0x040000
	statusBar()->setAutoFillBackground(TRUE);
	#endif
	statusBar()->setPaletteBackgroundColor( QColor(0,0,0) );
	statusBar()->setPaletteForegroundColor( QColor(255,255,255) );
	time_display->setPaletteBackgroundColor( QColor(0,0,0) );
	time_display->setPaletteForegroundColor( QColor(255,255,255) );
	frame_display->setPaletteBackgroundColor( QColor(0,0,0) );
	frame_display->setPaletteForegroundColor( QColor(255,255,255) );
	statusBar()->setSizeGripEnabled(FALSE);

    statusBar()->message( tr("Welcome to SMPlayer") );
	statusBar()->addWidget( frame_display, 0, TRUE );
	frame_display->setText( " 0 " );

    statusBar()->addWidget( time_display, 0, TRUE );
	time_display->setText("00:00:00 / 00:00:00");

	// panel
	#if QT_VERSION >= 0x040000
	panel->setAutoFillBackground(TRUE);
	#endif
	panel->setBackgroundColor( QColor(0,0,0) );

	// fullscreen_widget
	#if !FULLSCREEN_WITH_MAINWINDOW
	#if QT_VERSION >= 0x040000
	fullscreen_widget->setAutoFillBackground(TRUE);
	#endif
	fullscreen_widget->setBackgroundColor( QColor(0,0,0) );
	#endif

	pref_dialog = new PreferencesDialog(this, "pref_dialog", FALSE);
	connect( pref_dialog, SIGNAL(applyButtonPressed()),
             this, SLOT(applyNewPreferences()) );
	// Clear recent's list
	connect( pref_dialog, SIGNAL(clearRecentsButtonPressed()),
             this, SLOT(clearRecentsList()) );

	file_dialog = new FilePreferencesDialog(this, "file_dialog", FALSE);
	connect( file_dialog, SIGNAL(applyButtonPressed()),
             this, SLOT(applyFilePreferences()) );


	//createMenus();
	languageChange();
	connectWidgets();

	setAcceptDrops(TRUE);

	if (pref->compact_mode) toggleCompactMode(TRUE);
	if (pref->stay_on_top) toggleStayOnTop(TRUE);
	toggleFrameCounter( pref->show_frame_counter );

	recents->setMaxItems( pref->recents_max_items);
	//recents->load();
	updateRecents();

	if (pref->use_single_instance) {
		// Client which will try to connect to other running instance
		client = new Client(pref->connection_port, this, "client");
		connect( client, SIGNAL(connected()), this, SLOT(connectedToPort()) );
		connect( client, SIGNAL(notConnected()), this, SLOT(notConnectedToPort()) );
		connect( client, SIGNAL(serverIdentified()), 
	             this, SLOT(connectedToInstance()) );
		connect( client, SIGNAL(connectionClosed()), 
	             this, SLOT(connectionToInstanceClosed()) );
		connect( client, SIGNAL(unknownServer()),
	             this, SLOT(connectedToUnknownServer()) );
	} else {
		show();
		if (!initial_file.isEmpty()) {
			open(initial_file);
		}
	}
}

Gui::~Gui() {
	delete mplayer_log_window;
	delete smplayer_log_window;
	delete info_window;
	delete playlist;
}

void Gui::languageChange() {
	GuiBase::languageChange();

	mplayer_log_window->setCaption( tr("SMPlayer - mplayer log") );
	smplayer_log_window->setCaption( tr("SMPlayer - smplayer log") );

#if DOCK_PLAYLIST
	playlistdock->setCaption( tr("Playlist") );
#endif

	menuBar()->clear();
	createMenus();

	updateRecents();
	updateWidgets();
}

void Gui::connectedToPort() {
	qDebug("Gui::connectedToPort");
}

void Gui::notConnectedToPort() {
	qDebug("Gui::notConnectedToPort");

	server = new Server(pref->connection_port, this);
	connect(server, SIGNAL(receivedOpen(QString)),
            this, SLOT(remoteOpen(QString)));
	connect(server, SIGNAL(receivedFunction(QString)),
            this, SLOT(processFunction(QString)));
	show();

	if (!initial_file.isEmpty()) {
		open(initial_file);
	}

}

void Gui::connectedToInstance() {
	qDebug("Gui::connectedToInstance");

	if (!initial_file.isEmpty()) {
		//client->sendToServer("open " + initial_file);
		client->sendToServer("open_utf8 " + initial_file.utf8());
	}

	client->closeConnection();
}

void Gui::connectionToInstanceClosed() {
	qDebug("Gui::connectedToInstanceClosed");
	close();
}

void Gui::connectedToUnknownServer() {
	qDebug("Gui::connectedToUnknownServer");
	qDebug(" * Keeping running");
	show();

	if (!initial_file.isEmpty()) {
		open(initial_file);
	}
}

void Gui::remoteOpen(QString file) {
	if (isMinimized()) showNormal();
	raise();
	setActiveWindow();
	open(file);
}

void Gui::createMenus() {
	// MENUS
	openMenu = new QPopupMenu(this);
	menuBar()->insertItem( tr("&Open"), openMenu );
	playMenu = new QPopupMenu(this);
	menuBar()->insertItem( tr("&Play"), playMenu );
	videoMenu = new QPopupMenu(this);
	menuBar()->insertItem( tr("&Video"), videoMenu );
	audioMenu = new QPopupMenu(this);
	menuBar()->insertItem( tr("&Audio"), audioMenu );
	subtitlesMenu = new QPopupMenu( this );
	menuBar()->insertItem( tr("&Subtitles"), subtitlesMenu );
	browseMenu = new QPopupMenu(this);
	menuBar()->insertItem( tr("&Browse"), browseMenu );
	optionsMenu = new QPopupMenu( this );
	menuBar()->insertItem( tr("Op&tions"), optionsMenu );
	helpMenu = new QPopupMenu( this );
	menuBar()->insertItem( tr("&Help"), helpMenu );


	// OPEN MENU
	openMenu->insertItem( Images2::icon("open"), tr("&File..."), 
                          this, SLOT(openFile()), kl->find("OPEN_FILE") );
	openMenu->insertItem( Images2::icon("openfolder"), tr("D&irectory..."), 
                          this, SLOT(openDirectory()) );
	openMenu->insertItem( Images2::icon("open_playlist"), tr("&Playlist..."), 
                          playlist, SLOT(load()) );
	openMenu->insertItem( Images2::icon("dvd"), tr("&DVD from drive"), 
                          this, SLOT(openDVD()) );
	openMenu->insertItem( Images2::icon("dvd_hd"), tr("D&VD from folder..."), 
                          this, SLOT(openDVDFromFolder()) );
	openMenu->insertItem( Images2::icon("url"), tr("&URL..."), 
                          this, SLOT(openURL()), kl->find("OPEN_URL") );

	recentfiles_menu = new QPopupMenu;
	recentfiles_menu->insertItem( tr("&Clear"), this, SLOT(clearRecentsList()), 0, RECENTS_CLEAR );
	recentfiles_menu->insertSeparator();
	connect( recentfiles_menu, SIGNAL(activated(int)), 
             this, SLOT(openRecent(int)) );

	openMenu->insertItem( Images2::icon("recents"), tr("&Recent files"), recentfiles_menu );
	openMenu->insertSeparator();
	openMenu->insertItem( Images2::icon("exit"), tr("&Exit"), this, SLOT(close()), kl->find("EXIT") );
	
	// PLAY MENU
	playMenu->setCheckable(TRUE);
	playMenu->insertItem( Images2::icon("play"), tr("P&lay"), core, SLOT(play()) );
	playMenu->insertItem( Images2::icon("pause"), tr("&Pause"), core, SLOT(pause()), kl->find("PAUSE") );
	playMenu->insertItem( Images2::icon("stop"), tr("&Stop"), core, SLOT(stop()), kl->find("STOP") );
	playMenu->insertItem( Images2::icon("frame_step"), tr("&Frame step"), core, SLOT(frameStep()), kl->find("FRAME_STEP") );
	playMenu->insertSeparator();
	playMenu->insertItem( Images2::icon("rewind10s"), tr("&-10 seconds"), core, SLOT(srewind()), kl->find("REWIND1") );
	playMenu->insertItem( Images2::icon("forward10s"), tr("&+10 seconds"), core, SLOT(sforward()), kl->find("FORWARD1") );
	playMenu->insertSeparator();
	playMenu->insertItem( Images2::icon("rewind1m"), tr("-1 &minute"), core, SLOT(rewind()), kl->find("REWIND2") );
	playMenu->insertItem( Images2::icon("forward1m"), tr("+1 m&inute"), core, SLOT(forward()), kl->find("FORWARD2") );
	playMenu->insertSeparator();
	playMenu->insertItem( Images2::icon("rewind10m"), tr("-10 mi&nutes"), core, SLOT(fastrewind()), kl->find("REWIND3") );
	playMenu->insertItem( Images2::icon("forward10m"), tr("+10 min&utes"), core, SLOT(fastforward()), kl->find("FORWARD3") );
	playMenu->insertSeparator();

	// Speed submenu
	speed_menu = new QPopupMenu;
	speed_menu->insertItem( tr("&Normal speed"), core, SLOT(normalSpeed()), kl->find("NORMAL_SPEED") );
	speed_menu->insertItem( tr("&Halve speed"), core, SLOT(halveSpeed()), kl->find("HALVE_SPEED") );
	speed_menu->insertItem( tr("&Double speed"), core, SLOT(doubleSpeed()), kl->find("DOUBLE_SPEED") );
	speed_menu->insertItem( tr("Speed &-10%"), core, SLOT(decSpeed()), kl->find("DEC_SPEED") );
	speed_menu->insertItem( tr("Speed &+10%"), core, SLOT(incSpeed()), kl->find("INC_SPEED") );
	playMenu->insertItem( Images2::icon("speed"), tr("Sp&eed"), speed_menu );
	playMenu->insertItem( Images2::icon("repeat"), tr("&Repeat"), 
                          core, SLOT(toggleRepeat()), 0, PLAY_REPEAT );
	
	// VIDEO MENU
	videoMenu->setCheckable(TRUE);
	videoMenu->insertItem( Images2::icon("fullscreen"), tr("&Fullscreen"), this, 
                           SLOT(toggleFullscreen()), kl->find("FULLSCREEN") );
	videoMenu->insertItem( Images2::icon("compact"), tr("&Compact mode"), this, 
                            SLOT(toggleCompactMode()), kl->find("COMPACT_MODE"), VIDEO_COMPACT_MODE );

	// Size submenu
	videosize_menu = new QPopupMenu;
	videosize_menu->insertItem( tr("5&0%"), 50 );
	videosize_menu->insertItem( tr("7&5%"), 75 );
	videosize_menu->insertItem( tr("&100%"), 100 );
	videosize_menu->insertItem( tr("1&25%"), 125 );
	videosize_menu->insertItem( tr("15&0%"), 150 );
	videosize_menu->insertItem( tr("1&75%"), 175 );
	videosize_menu->insertItem( tr("&200%"), 200 );
	videosize_menu->insertItem( tr("&300%"), 300 );
	videosize_menu->insertItem( tr("&400%"), 400 );
	videosize_menu->setCheckable(TRUE);
	connect( videosize_menu, SIGNAL(activated(int)),
             core, SLOT(changeZoom(int)) );
	videoMenu->insertItem( Images2::icon("video_size"), tr("Si&ze"), videosize_menu );

	// Aspect submenu
	aspect_menu = new QPopupMenu;
	aspect_menu->insertItem( tr("&Autodetect"), ASPECT_AUTO );
	aspect_menu->insertItem( tr("&4:3"), ASPECT_43 );
	aspect_menu->insertItem( tr("&5:4"), ASPECT_54 );
	aspect_menu->insertItem( tr("&14:9"), ASPECT_149 );
	aspect_menu->insertItem( tr("16:&9"), ASPECT_169 );
	aspect_menu->insertItem( tr("1&6:10"), ASPECT_1610 );
	aspect_menu->insertItem( tr("&2.35:1"), ASPECT_235 );
	aspect_menu->insertSeparator();
	aspect_menu->insertItem( tr("4:3 &Letterbox"), ASPECT_43_LETTERBOX );
	aspect_menu->insertItem( tr("16:9 L&etterbox"), ASPECT_169_LETTERBOX );
	aspect_menu->insertItem( tr("4:3 &Panscan"), ASPECT_43_PANSCAN );
	aspect_menu->insertItem( tr("4:3 &to 16:9"), ASPECT_43_TO_169 );
	aspect_menu->setCheckable(TRUE);

	videoMenu->insertItem( Images2::icon("aspect"), tr("&Aspect ratio"), aspect_menu );
	connect( aspect_menu, SIGNAL(activated(int)),
             core, SLOT(changeAspectRatio(int)) );

	// Deinterlace submenu
	deinterlace_menu = new QPopupMenu;
	deinterlace_menu->insertItem( tr("&None"), DEINTERLACE_NONE );
	deinterlace_menu->insertItem( tr("&Lowpass5"), DEINTERLACE_L5 );
	deinterlace_menu->insertItem( tr("&Yadif"), DEINTERLACE_YADIF );
	deinterlace_menu->insertItem( tr("Linear &Blend"), DEINTERLACE_LB );

	videoMenu->insertItem( Images2::icon("deinterlace"), tr("&Deinterlace"), deinterlace_menu );
	connect( deinterlace_menu, SIGNAL(activated(int)),
             core, SLOT(changeDeinterlace(int)) );

	// Video filter submenu
	videofilter_menu = new QPopupMenu;
	videofilter_menu->setCheckable(TRUE);
	videofilter_menu->insertItem( tr("&Postprocessing"), core, SLOT(togglePostprocessing()), 0, VIDEO_FILTER_POSTPROCESSING );
	videofilter_menu->insertItem( tr("&Autodetect phase"), core, SLOT(toggleAutophase()), 0, VIDEO_FILTER_AUTOPHASE );
	videofilter_menu->insertItem( tr("Denoise (&normal)"), core, SLOT(toggleDenoiseNormal()), 0, VIDEO_FILTER_DENOISE_NORMAL );
	videofilter_menu->insertItem( tr("Denoise (&soft)"), core, SLOT(toggleDenoiseSoft()), 0, VIDEO_FILTER_DENOISE_SOFT );
	videofilter_menu->insertItem( tr("&Deblock"), core, SLOT(toggleDeblock()), 0, VIDEO_FILTER_DEBLOCK );
	videofilter_menu->insertItem( tr("De&ring"), core, SLOT(toggleDering()), 0, VIDEO_FILTER_DERING );
	videofilter_menu->insertItem( tr("Add n&oise"), core, SLOT(toggleNoise()), 0, VIDEO_FILTER_NOISE );
	videoMenu->insertItem( Images2::icon("video_filters"), tr("F&ilters"), videofilter_menu );
	/*
	videoMenu->insertSeparator();
	videoMenu->insertItem( tr("Contrast -"), core, SLOT(decContrast()), kl->find("DEC_CONTRAST") );
	videoMenu->insertItem( tr("Contrast +"), core, SLOT(incContrast()), kl->find("INC_CONTRAST") );
	videoMenu->insertSeparator();
	videoMenu->insertItem( tr("Brightness -"), core, SLOT(decBrightness()), kl->find("DEC_BRIGHTNESS") );
	videoMenu->insertItem( tr("Brightness +"), core, SLOT(incBrightness()), kl->find("INC_BRIGHTNESS") );
	videoMenu->insertSeparator();
	videoMenu->insertItem( tr("Hue -"), core, SLOT(decHue()), kl->find("DEC_HUE") );
	videoMenu->insertItem( tr("Hue +"), core, SLOT(incHue()), kl->find("INC_HUE") );
	videoMenu->insertSeparator();
	videoMenu->insertItem( tr("Saturation -"), core, SLOT(decSaturation()), kl->find("DEC_SATURATION") );
	videoMenu->insertItem( tr("Saturation +"), core, SLOT(incSaturation()), kl->find("INC_SATURATION") );
	videoMenu->insertSeparator();
	videoMenu->insertItem( tr("Gamma -"), core, SLOT(decGamma()), kl->find("DEC_GAMMA") );
	videoMenu->insertItem( tr("Gamma +"), core, SLOT(incGamma()), kl->find("INC_GAMMA") );
	videoMenu->insertSeparator();
	*/
	videoMenu->insertItem( Images2::icon("equalizer"), tr("&Equalizer"), this, SLOT(showEqualizer()), kl->find("VIDEO_EQUALIZER"), VIDEO_EQUALIZER );
	/*videoMenu->insertSeparator();*/
	videoMenu->insertItem( Images2::icon("screenshot"), tr("&Screenshot"), 
                           core, SLOT(screenshot()), kl->find("SCREENSHOT") );
	videoMenu->insertItem( Images2::icon("ontop"), tr("S&tay on top"), this, SLOT(toggleStayOnTop()), 
                           kl->find("STAY_ON_TOP"), VIDEO_STAY_ON_TOP );

    // AUDIO MENU
	audioMenu->setCheckable(TRUE);
	// Audio track submenu
	audiotrack_menu = new QPopupMenu;
    audiotrack_menu->setCheckable(TRUE);
    connect( audiotrack_menu, SIGNAL(activated(int)), 
	        core, SLOT(changeAudio(int)) );
    audioMenu->insertItem( Images2::icon("audio_track"), tr("&Track"), audiotrack_menu, -1, 0);

	// Filter submenu
	audiofilter_menu = new QPopupMenu;
	audiofilter_menu->setCheckable(TRUE);
	audiofilter_menu->insertItem( tr("&Extrastereo"), core, SLOT(toggleExtrastereo()), 0, AUDIO_FILTER_EXTRASTEREO );
	audiofilter_menu->insertItem( tr("&Karaoke"), core, SLOT(toggleKaraoke()), 0, AUDIO_FILTER_KARAOKE );
	audioMenu->insertItem( Images2::icon("audio_filters"), tr("&Filters"), audiofilter_menu );

	audiochannels_menu = new QPopupMenu;
	audiochannels_menu->setCheckable(TRUE);
	audiochannels_menu->insertItem( tr("&Default"), AUDIO_CHANNELS_DEFAULT );
	audiochannels_menu->insertItem( tr("&Stereo"), AUDIO_CHANNELS_STEREO );
	audiochannels_menu->insertItem( tr("&4.0 Surround"), AUDIO_CHANNELS_SURROUND );
	audiochannels_menu->insertItem( tr("&5.1 Surround"), AUDIO_CHANNELS_FULL5_1 );
	audioMenu->insertItem( Images2::icon("audio_channels"), tr("&Channels") , audiochannels_menu );
	connect( audiochannels_menu, SIGNAL(activated(int)),
             core, SLOT(setAudioChannels(int)) );

	stereomode_menu = new QPopupMenu;
	stereomode_menu->setCheckable(TRUE);
	stereomode_menu->insertItem( tr("&Stereo"), AUDIO_STEREO_STEREO );
	stereomode_menu->insertItem( tr("&Left channel"), AUDIO_STEREO_LEFT );
	stereomode_menu->insertItem( tr("&Right channel"), AUDIO_STEREO_RIGHT );
	audioMenu->insertItem( Images2::icon("stereo_mode"), tr("&Stereo mode"), stereomode_menu );
	connect( stereomode_menu, SIGNAL(activated(int)),
             core, SLOT(setStereoMode(int)) );

	audioMenu->insertItem( Images2::icon("mute"), tr("&Mute"), core, SLOT(switchMute()), 
                           kl->find("MUTE"), AUDIO_MUTE);
	audioMenu->insertSeparator();
	audioMenu->insertItem( Images2::icon("audio_down"), tr("Volume &-"), core, SLOT(decVolume()), kl->find("DEC_VOLUME"), AUDIO_VOL_DEC );
	audioMenu->insertItem( Images2::icon("audio_up"), tr("Volume &+"), core, SLOT(incVolume()), kl->find("INC_VOLUME"), AUDIO_VOL_INC );
	audioMenu->insertSeparator();
	audioMenu->insertItem( Images2::icon("delay_down"), tr("&Delay -"), core, SLOT(decAudioDelay()), kl->find("DEC_AUDIO_DELAY") );
	audioMenu->insertItem( Images2::icon("delay_up"), tr("D&elay +"), core, SLOT(incAudioDelay()), kl->find("INC_AUDIO_DELAY") );


	// SUBTITLES MENU
	// Track submenu
    subtitlestrack_menu = new QPopupMenu;
	subtitlestrack_menu->setCheckable(TRUE);
    connect( subtitlestrack_menu, SIGNAL(activated(int)), 
	         core, SLOT(changeSubtitle(int)) );
    subtitlesMenu->insertItem( Images2::icon("sub"), tr("&Select"), subtitlestrack_menu, -1, 0);
    subtitlesMenu->insertItem( Images2::icon("open"), tr("&Load..."), this, SLOT(loadSub()) );
	subtitlesMenu->insertSeparator();
	subtitlesMenu->insertItem( Images2::icon("delay_down"), tr("Delay &-"), core, SLOT(decSubDelay()), kl->find("DEC_SUB_DELAY") );
	subtitlesMenu->insertItem( Images2::icon("delay_up"), tr("Delay &+"), core, SLOT(incSubDelay()), kl->find("INC_SUB_DELAY") );
	subtitlesMenu->insertSeparator();
	subtitlesMenu->insertItem( Images2::icon("sub_up"), tr("&Up"), core, SLOT(decSubPos()), kl->find("DEC_SUBPOS") );
	subtitlesMenu->insertItem( Images2::icon("sub_down"), tr("&Down"), core, SLOT(incSubPos()), kl->find("INC_SUBPOS") );


	// BROWSE MENU
	// Titles submenu
	titles_menu = new QPopupMenu;
	titles_menu->setCheckable(TRUE);
	connect( titles_menu, SIGNAL(activated(int)),
			core, SLOT(changeTitle(int)) );
	browseMenu->insertItem( Images2::icon("title"), tr("&Title"), titles_menu );

	// Chapters submenu
	chapters_menu = new QPopupMenu;
	chapters_menu->setCheckable(TRUE);
	connect( chapters_menu, SIGNAL(activated(int)),
			core, SLOT(changeChapter(int)) );
	browseMenu->insertItem( Images2::icon("chapter"), tr("&Chapter"), chapters_menu );

	// Angles submenu
	angles_menu= new QPopupMenu;
	angles_menu->setCheckable(TRUE);
	connect( angles_menu, SIGNAL(activated(int)),
			core, SLOT(changeAngle(int)) );
	browseMenu->insertItem( Images2::icon("angle"), tr("&Angle"), angles_menu );


	// OPTIONS MENU
	optionsMenu->insertItem( Images2::icon("info"), tr("View &info"), this, SLOT(showInfo()), kl->find("INFO") );
	optionsMenu->insertItem( Images2::icon("playlist"), tr("&Playlist"), this, SLOT(showPlaylist()), kl->find("PLAYLIST"), OPTIONS_PLAYLIST );
	optionsMenu->insertItem( Images2::icon("file_codecs"), tr("&Advanced media settings..."), this, SLOT(showFilePreferencesDialog()) );
	optionsMenu->insertItem( Images2::icon("frame_counter"), tr("&Show frame counter"), this, SLOT(toggleFrameCounter()), 0, OPTIONS_FRAMECOUNTER );

	// OSD submenu
	osd_menu = new QPopupMenu;
	osd_menu->insertItem( tr("&Disabled"), OSD_NONE );
	osd_menu->insertItem( tr("&Seek bar"), OSD_SEEK );
	osd_menu->insertItem( tr("&Time"), OSD_SEEK_TIMER );
	osd_menu->insertItem( tr("Time + T&otal time"), OSD_SEEK_TIMER_TOTAL );
	optionsMenu->insertItem( Images2::icon("osd"), tr("&OSD"), osd_menu );
	connect( osd_menu, SIGNAL(activated(int)),
             core, SLOT(changeOSD(int)) );

	// Logs submenu
	logs_menu = new QPopupMenu;
	logs_menu->insertItem( "mplayer", this, SLOT(showMplayerLog()), kl->find("MPLAYER_LOG") );
	logs_menu->insertItem( "smplayer", this, SLOT(showLog()), kl->find("SMPLAYER_LOG") );
	optionsMenu->insertItem( Images2::icon("logs"), tr("&View logs"), logs_menu );
	optionsMenu->insertItem( Images2::icon("prefs"), tr("P&references"), this, SLOT( showPreferencesDialog() ), kl->find("PREFERENCES") );


	// HELP MENU
	helpMenu->insertItem( Images2::icon("qt"), tr("About &Qt"), this, SLOT(helpAboutQt()) );
	helpMenu->insertItem( Images2::icon("logo_small"), 
                          tr("About &SMPlayer"), this, SLOT(helpAbout()) );

	// POPUP MENU
	popup = new QPopupMenu;
	popup->insertItem( tr("&Open"), openMenu );
	popup->insertItem( tr("&Play"), playMenu );
	popup->insertItem( tr("&Video"), videoMenu );
	popup->insertItem( tr("&Audio"), audioMenu );
	popup->insertItem( tr("&Subtitles"), subtitlesMenu );
	popup->insertItem( tr("&Browse"), browseMenu );
	popup->insertItem( tr("Op&tions"), optionsMenu );

	// let's show something, even a <empty> entry
	initializeMenus();
}

void Gui::connectWidgets() {
	connect( fastrewindbutton, SIGNAL( clicked() ), core, SLOT( fastrewind() ) );
	connect( rewindbutton, SIGNAL( clicked() ), core, SLOT( rewind() ) );
	connect( smallrewindbutton, SIGNAL( clicked() ), core, SLOT( srewind() ) );

	connect( timeslider, SIGNAL( posChanged(int) ), core, SLOT(goToPos(int)) );
	connect( timeslider, SIGNAL( draggingPos(int) ), this, SLOT(displayGotoTime(int)) );

	connect( smallforwardbutton, SIGNAL( clicked() ), core, SLOT( sforward() ) );
	connect( forwardbutton, SIGNAL( clicked() ), core, SLOT( forward() ) );
	connect( fastforwardbutton, SIGNAL( clicked() ), core, SLOT( fastforward() ) );

	connect( playbutton, SIGNAL( clicked() ), core, SLOT( play() ) );
	connect( pausebutton, SIGNAL( clicked() ), core, SLOT( pause_and_frame_step() ) );
	connect( stopbutton, SIGNAL( clicked() ), core, SLOT( stop() ) );
	connect( fullscreenbutton, SIGNAL( toggled(bool) ), this, SLOT( toggleFullscreen(bool) ) );
	connect( mutebutton, SIGNAL( clicked() ), core, SLOT( switchMute() ) );
	connect( volumeslider, SIGNAL( valueChanged(int) ), core, SLOT( setVolume(int) ) );

	connect( playlistbutton, SIGNAL( clicked() ), this, SLOT( showPlaylist() ) );

	connect( prevbutton, SIGNAL( clicked() ), playlist, SLOT( playPrev() ) );
	connect( nextbutton, SIGNAL( clicked() ), playlist, SLOT( playNext() ) );

	// Equalizer
	connect( equalizer->contrast, SIGNAL(valueChanged(int)), 
             core, SLOT(setContrast(int)) );
	connect( equalizer->brightness, SIGNAL(valueChanged(int)), 
             core, SLOT(setBrightness(int)) );
	connect( equalizer->hue, SIGNAL(valueChanged(int)), 
             core, SLOT(setHue(int)) );
	connect( equalizer->saturation, SIGNAL(valueChanged(int)), 
             core, SLOT(setSaturation(int)) );
	connect( equalizer->gamma, SIGNAL(valueChanged(int)), 
             core, SLOT(setGamma(int)) );
}


void Gui::closeEvent( QCloseEvent * e )  {
	qDebug("Gui::closeEvent");

	qDebug("mplayer_log_window: %d x %d", mplayer_log_window->width(), mplayer_log_window->height() );
	qDebug("smplayer_log_window: %d x %d", smplayer_log_window->width(), smplayer_log_window->height() );
	qDebug("info_window: %d x %d", info_window->width(), info_window->height() );

	mplayer_log_window->close();
	smplayer_log_window->close();
	info_window->close();
	playlist->close();
	equalizer->close();
	core->stop();
	e->accept();
}

void Gui::showPlaylist() {
	if (playlist->isVisible()) {
		#if DOCK_PLAYLIST
		playlistdock->hide();
		#else
		playlist->hide();
		#endif
		playlistbutton->setOn(FALSE);
	} else {
		exitFullscreenIfNeeded();
		#if DOCK_PLAYLIST
		playlistdock->show();
		#else
		playlist->show();
		#endif
		playlistbutton->setOn(TRUE);
	}
	updateWidgets();
}

void Gui::showEqualizer() {
	if (equalizer->isVisible()) {
		equalizer->hide();
	} else {
		// Exit fullscreen, otherwise dialog is not visible
		exitFullscreenIfNeeded();
		equalizer->show();
	}
	updateWidgets();
}

void Gui::showPreferencesDialog() {
	qDebug("Gui::showPreferencesDialog");

	exitFullscreenIfNeeded();
	
	if ( (!file_dialog->hasCodecsList()) || (!pref_dialog->hasDriversList()) ) {
		InfoReader ir( pref->mplayer_bin );
		ir.getInfo();
		file_dialog->setCodecs( ir.vcList(), ir.acList(), ir.demuxerList() );
		pref_dialog->setDrivers( ir.voList(), ir.aoList() );
	}

	pref_dialog->setMplayerPath( pref->mplayer_bin );
	pref_dialog->setScreenshotDir( pref->screenshot_directory );
	pref_dialog->setVO( pref->vo );
	pref_dialog->setAO( pref->ao );
	pref_dialog->setEq2( pref->use_soft_video_eq );
	pref_dialog->setSoftVol( pref->use_soft_vol );
	pref_dialog->setRememberSettings( !pref->dont_remember_media_settings );
	pref_dialog->setDontRememberTimePos( pref->dont_remember_time_pos );
	pref_dialog->setStartInFullscreen( pref->start_in_fullscreen );
	pref_dialog->setAudioLang( pref->audio_lang );
	pref_dialog->setSubtitlesOnScreenshots( pref->subtitles_on_screenshots );

	pref_dialog->setDVDDevice( pref->dvd_device );
	pref_dialog->setCDRomDevice( pref->cdrom_device );

	pref_dialog->setDisableScreensaver( pref->disable_screensaver );
	pref_dialog->setResizeMethod( pref->resize_method );
	//pref_dialog->setWheelSeek( pref->wheel_seek );
	pref_dialog->setMonitorAspect( pref->monitor_aspect );

	pref_dialog->setUseSingleInstance(pref->use_single_instance);
	pref_dialog->setServerPort(pref->connection_port);
	pref_dialog->setRecentsMaxItems(pref->recents_max_items);

	pref_dialog->setSeeking1(pref->seeking1);
	pref_dialog->setSeeking2(pref->seeking2);
	pref_dialog->setSeeking3(pref->seeking3);
	pref_dialog->setSeeking4(pref->seeking4);

	pref_dialog->setLeftClickFunction( pref->left_click_function );
	pref_dialog->setDoubleClickFunction( pref->double_click_function );
	pref_dialog->setWheelChangesVolume( pref->wheel_changes_volume );
	pref_dialog->setInitialVolume( pref->initial_volume );

	pref_dialog->setLanguage( pref->language );
	pref_dialog->setIconSet( pref->iconset );

	pref_dialog->setFontName( pref->font_name );
	pref_dialog->setFontFile( pref->font_file );
	pref_dialog->setUseFontconfig( pref->use_fontconfig );
	pref_dialog->setFontAutoscale( pref->font_autoscale );
	pref_dialog->setFontTextscale( pref->font_textscale );
	pref_dialog->setAutoloadSub( pref->autoload_sub );
	pref_dialog->setFontFuzziness( pref->subfuzziness );
	pref_dialog->setFontEncoding( pref->subcp );
	pref_dialog->setUseFontASS( pref->use_ass_subtitles );
	pref_dialog->setAssColor( pref->ass_color );
	pref_dialog->setAssBorderColor( pref->ass_border_color );

	pref_dialog->setUseMplayerWindow( pref->use_mplayer_window );
	pref_dialog->setCache( pref->cache );
	pref_dialog->setMplayerAdditionalArguments( pref->mplayer_additional_options );
	pref_dialog->setMplayerAdditionalVideoFilters( pref->mplayer_additional_video_filters );
	pref_dialog->setMplayerAdditionalAudioFilters( pref->mplayer_additional_audio_filters );

	pref_dialog->setLogMplayer( pref->log_mplayer );
	pref_dialog->setLogSmplayer( pref->log_smplayer );
	pref_dialog->setLogFilter( pref->log_filter );

	pref_dialog->setPriority( pref->priority );
	pref_dialog->setFrameDrop( pref->frame_drop );
	pref_dialog->setHardFrameDrop( pref->hard_frame_drop );
	pref_dialog->setAutoq( pref->autoq );
	pref_dialog->setAutoSyncActivated( pref->autosync );
	pref_dialog->setAutoSyncFactor( pref->autosync_factor );
	pref_dialog->setFastChapterSeeking( pref->fast_chapter_change );
	pref_dialog->setFastAudioSwitching( !pref->audio_change_requires_restart );

#if STYLE_SWITCHING
	pref_dialog->setStyle( pref->style );
#endif

	pref_dialog->show();
}

// The user has pressed OK in preferences dialog
void Gui::applyNewPreferences() {
	qDebug("Gui::applyNewPreferences");

	bool need_restart = FALSE;

#define TEST_AND_SET( Pref, Dialog ) \
	if ( Pref != Dialog ) { Pref = Dialog; need_restart = TRUE; }

	TEST_AND_SET(pref->mplayer_bin, pref_dialog->mplayerPath());
	TEST_AND_SET(pref->screenshot_directory, pref_dialog->screenshotDir());
	TEST_AND_SET(pref->vo, pref_dialog->VO());
	TEST_AND_SET(pref->ao, pref_dialog->AO());
	TEST_AND_SET(pref->use_soft_video_eq, pref_dialog->eq2());
	TEST_AND_SET(pref->use_soft_vol, pref_dialog->softVol());
	bool dont_remember_ms = !pref_dialog->rememberSettings();
	TEST_AND_SET(pref->dont_remember_media_settings, dont_remember_ms);
	TEST_AND_SET(pref->dont_remember_time_pos, pref_dialog->dontRememberTimePos());
	pref->start_in_fullscreen = pref_dialog->startInFullscreen();
	pref->audio_lang = pref_dialog->audioLang();
	TEST_AND_SET(pref->subtitles_on_screenshots, pref_dialog->subtitlesOnScreenshots());

	pref->dvd_device = pref_dialog->dvdDevice();
	pref->cdrom_device = pref_dialog->cdromDevice();

	TEST_AND_SET(pref->disable_screensaver, pref_dialog->disableScreensaver());
	pref->resize_method = pref_dialog->resizeMethod();
	TEST_AND_SET(pref->monitor_aspect, pref_dialog->monitorAspect());
	mplayerwindow->setMonitorAspect( pref->monitor_aspect_double() );

	pref->use_single_instance = pref_dialog->useSingleInstance();
	pref->connection_port = pref_dialog->serverPort();
	pref->recents_max_items = pref_dialog->recentsMaxItems();
	recents->setMaxItems(pref->recents_max_items);
	updateRecents();

	pref->seeking1 = pref_dialog->seeking1();
	pref->seeking2 = pref_dialog->seeking2();
	pref->seeking3 = pref_dialog->seeking3();
	pref->seeking4 = pref_dialog->seeking4();

	pref->left_click_function = pref_dialog->leftClickFunction();
	pref->double_click_function = pref_dialog->doubleClickFunction();
	pref->wheel_changes_volume = pref_dialog->wheelChangesVolume();
	pref->initial_volume = pref_dialog->initialVolume();

	if (pref->language != pref_dialog->language()) {
		pref->language = pref_dialog->language();
		translator->load(pref->language);
	}

	if (pref->iconset != pref_dialog->iconSet()) {
		pref->iconset = pref_dialog->iconSet();
		translator->load(pref->language); // Not a mistake
	}

	TEST_AND_SET(pref->font_name, pref_dialog->fontName());
	TEST_AND_SET(pref->font_file, pref_dialog->fontFile());
	TEST_AND_SET(pref->use_fontconfig, pref_dialog->useFontconfig());
	TEST_AND_SET(pref->font_autoscale, pref_dialog->fontAutoscale());
	TEST_AND_SET(pref->font_textscale, pref_dialog->fontTextscale());
	TEST_AND_SET(pref->autoload_sub, pref_dialog->autoloadSub());
	TEST_AND_SET(pref->subfuzziness, pref_dialog->fontFuzziness());
	TEST_AND_SET(pref->subcp, pref_dialog->fontEncoding());
	TEST_AND_SET(pref->use_ass_subtitles, pref_dialog->useFontASS());
	TEST_AND_SET(pref->ass_color, pref_dialog->assColor());
	TEST_AND_SET(pref->ass_border_color, pref_dialog->assBorderColor());

	TEST_AND_SET(pref->use_mplayer_window, pref_dialog->useMplayerWindow());
	TEST_AND_SET(pref->cache, pref_dialog->cache());
	TEST_AND_SET(pref->mplayer_additional_options, pref_dialog->mplayerAdditionalArguments());
	TEST_AND_SET(pref->mplayer_additional_video_filters, pref_dialog->mplayerAdditionalVideoFilters());
	TEST_AND_SET(pref->mplayer_additional_audio_filters, pref_dialog->mplayerAdditionalAudioFilters());

	pref->log_mplayer = pref_dialog->logMplayer();
	pref->log_smplayer = pref_dialog->logSmplayer();
	pref->log_filter = pref_dialog->logFilter();

	TEST_AND_SET(pref->priority, pref_dialog->priority());
	TEST_AND_SET(pref->frame_drop, pref_dialog->frameDrop());
	TEST_AND_SET(pref->hard_frame_drop, pref_dialog->hardFrameDrop());
	TEST_AND_SET(pref->autoq, pref_dialog->autoq());
	TEST_AND_SET(pref->autosync, pref_dialog->autoSyncActivated());
	TEST_AND_SET(pref->autosync_factor, pref_dialog->autoSyncFactor());
	TEST_AND_SET(pref->fast_chapter_change, pref_dialog->fastChapterSeeking());
	bool audio_requires_restart = !pref_dialog->fastAudioSwitching();
	TEST_AND_SET(pref->audio_change_requires_restart, audio_requires_restart);

#if STYLE_SWITCHING
	pref->style = pref_dialog->style();
	qDebug( "selected style: '%s'", pref->style.utf8().data() );
	if ( !pref->style.isEmpty()) {
		qApp->setStyle( pref->style );
	} else {
		qDebug("setting default style: '%s'", default_style.utf8().data() );
		qApp->setStyle( default_style );
	}
#endif

	// Restart the video to apply
	if (need_restart)
		core->restart();

	pref->save();
}


void Gui::showFilePreferencesDialog() {
	qDebug("Gui::showFilePreferencesDialog");

	exitFullscreenIfNeeded();

	if ( (!file_dialog->hasCodecsList()) || (!pref_dialog->hasDriversList()) ) {
		InfoReader ir( pref->mplayer_bin );
		ir.getInfo();
		file_dialog->setCodecs( ir.vcList(), ir.acList(), ir.demuxerList() );
		pref_dialog->setDrivers( ir.voList(), ir.aoList() );
	}

	// Save a copy of the original values
	if (core->mset.original_demuxer.isEmpty()) 
		core->mset.original_demuxer = core->mdat.demuxer;

	if (core->mset.original_video_codec.isEmpty()) 
		core->mset.original_video_codec = core->mdat.video_codec;

	if (core->mset.original_audio_codec.isEmpty()) 
		core->mset.original_audio_codec = core->mdat.audio_codec;

	QString demuxer = core->mset.forced_demuxer;
	if (demuxer.isEmpty()) demuxer = core->mdat.demuxer;

	QString ac = core->mset.forced_audio_codec;
	if (ac.isEmpty()) ac = core->mdat.audio_codec;

	QString vc = core->mset.forced_video_codec;
	if (vc.isEmpty()) vc = core->mdat.video_codec;

	file_dialog->setDemuxer(demuxer, core->mset.original_demuxer);
	file_dialog->setAudioCodec(ac, core->mset.original_audio_codec);
	file_dialog->setVideoCodec(vc, core->mset.original_video_codec);

	file_dialog->setMplayerAdditionalArguments( core->mset.mplayer_additional_options );
	file_dialog->setMplayerAdditionalVideoFilters( core->mset.mplayer_additional_video_filters );
	file_dialog->setMplayerAdditionalAudioFilters( core->mset.mplayer_additional_audio_filters );

	file_dialog->show();
}

void Gui::applyFilePreferences() {
	qDebug("Gui::applyFilePreferences");

	QString demuxer = file_dialog->demuxer();
	if (demuxer == core->mset.original_demuxer) demuxer="";
	core->mset.forced_demuxer = demuxer;

	QString ac = file_dialog->audioCodec();
	if (ac == core->mset.original_audio_codec) ac="";
	core->mset.forced_audio_codec = ac;

	QString vc = file_dialog->videoCodec();
	if (vc == core->mset.original_video_codec) vc="";
	core->mset.forced_video_codec = vc;

	core->mset.mplayer_additional_options = file_dialog->mplayerAdditionalArguments();
	core->mset.mplayer_additional_video_filters = file_dialog->mplayerAdditionalVideoFilters();
	core->mset.mplayer_additional_audio_filters = file_dialog->mplayerAdditionalAudioFilters();

	// Restart the video to apply
	core->restart();
}

void Gui::showInfo() {
    qDebug("Gui::showInfo");

	exitFullscreenIfNeeded();
	
	info_window->setData( core->mdat );
	info_window->view->setCursorPosition(0,0);
	info_window->show();
}

void Gui::updateMediaInfo() {
    qDebug("Gui::updateMediaInfo");
	info_window->setData( core->mdat );

	setCaption( "SMPlayer - " + core->mdat.displayName() );

	recents->add( core->mdat.filename );
	updateRecents();
}

void Gui::showMplayerLog() {
    qDebug("Gui::showMplayerLog");

	exitFullscreenIfNeeded();

    mplayer_log_window->setText( core->mplayer_log );
	mplayer_log_window->show();
}

void Gui::showLog() {
    qDebug("Gui::showLog");

	exitFullscreenIfNeeded();

	smplayer_log_window->setText( Helper::log() );
    smplayer_log_window->show();
}


void Gui::initializeMenus() {
	qDebug("Gui::initializeMenus");

#define EMPTY 1

	int n;

	// Subtitles
    subtitlestrack_menu->clear();
    subtitlestrack_menu->insertItem( tr("&None"), SUB_ID_NONE );
	for (n=0; n < core->mdat.subtitles.numItems(); n++) {
		qDebug("Gui::initializeMenus: n: %d", n);
		subtitlestrack_menu->insertItem( core->mdat.subtitles.itemAt(n).displayName(), core->mdat.subtitles.itemAt(n).ID() );
	}

	// Audio
	audiotrack_menu->clear();
	if (core->mdat.audios.numItems()==0) {
		audiotrack_menu->insertItem( tr("<empty>"), EMPTY );
		audiotrack_menu->setItemEnabled(EMPTY, FALSE);
	} else {
		for (n=0; n < core->mdat.audios.numItems(); n++) {
			audiotrack_menu->insertItem( core->mdat.audios.itemAt(n).displayName(), core->mdat.audios.itemAt(n).ID() );
		}
	}

	// Titles
	titles_menu->clear();
	if (core->mdat.titles.numItems()==0) {
		titles_menu->insertItem( tr("<empty>"), EMPTY );
		titles_menu->setItemEnabled(EMPTY, FALSE);
	} else {
		for (n=0; n < core->mdat.titles.numItems(); n++) {
			titles_menu->insertItem( core->mdat.titles.itemAt(n).displayName(), core->mdat.titles.itemAt(n).ID() );
		}
	}

	// DVD Chapters
	chapters_menu->clear();
	if (core->mset.current_title_id > 0) {
		for (n=1; n <= core->mdat.titles.item(core->mset.current_title_id).chapters(); n++) {
			chapters_menu->insertItem( QString::number( n ), n );
		}
	} else {
		// *** Matroshka chapters ***
		if (core->mdat.mkv_chapters > 0) {
			for (n=0; n <= core->mdat.mkv_chapters; n++) {
				chapters_menu->insertItem( QString::number( n+1 ), n );
			}
		} else {
			chapters_menu->insertItem( tr("<empty>"), EMPTY );
			chapters_menu->setItemEnabled(EMPTY, FALSE);
		}
	}

	// Angles
	angles_menu->clear();
	if (core->mset.current_angle_id > 0) {
		for (n=1; n <= core->mdat.titles.item(core->mset.current_title_id).angles(); n++) {
			angles_menu->insertItem( QString::number( n ), n );
		}
	} else {
		angles_menu->insertItem( tr("<empty>"), EMPTY );
		angles_menu->setItemEnabled(EMPTY, FALSE);
	}
}

void Gui::updateRecents() {
	qDebug("Gui::updateRecents");

	//recentfiles_menu->clear();
	// Not clear the first 2 items
	while (recentfiles_menu->count() > 2) 
		recentfiles_menu->removeItemAt(2);

	if (recents->count() > 0) {
		int max_items = recents->count();
		if (max_items > pref->recents_max_items) {
			max_items=pref->recents_max_items;
		}
		for (int n=0; n < max_items; n++) {
			QString file = recents->item(n);
			QFileInfo fi(file);
			if (fi.exists()) file = fi.fileName();
			recentfiles_menu->insertItem( file, n );
		}
	} else {
		recentfiles_menu->insertItem( tr("<empty>"), EMPTY );
		recentfiles_menu->setItemEnabled(EMPTY, FALSE);
	}
}

void Gui::clearRecentsList() {
	// Delete items in menu
	/*
	for (int n=0; n < recents->count(); n++) {
		qDebug("borrando item %d de %d", n, recents->count());
		recentfiles_menu->removeItem(n);
	}
	*/

	recents->clear();
	updateRecents();
}

void Gui::updateWidgets() {
	qDebug("Gui::updateWidgets");

	// Subtitles menu
	Helper::checkMenu( subtitlestrack_menu, core->mset.current_sub_id );

	// If using an external subtitles, disable the rest
	bool b = core->mset.external_subtitles.isEmpty();
    for (unsigned int n=1; n < subtitlestrack_menu->count(); n++) {
		subtitlestrack_menu->setItemEnabled(subtitlestrack_menu->idAt(n),b);
	}
	
	// Audio menu
	Helper::checkMenu(audiotrack_menu, core->mset.current_audio_id );
	Helper::checkMenu(audiochannels_menu, core->mset.audio_use_channels );
	Helper::checkMenu(stereomode_menu, core->mset.stereo_mode );

	// Aspect ratio
	Helper::checkMenu(aspect_menu, core->mset.aspect_ratio_id );

	// OSD
	Helper::checkMenu(osd_menu, pref->osd );

	// Titles
	Helper::checkMenu(titles_menu, core->mset.current_title_id);

	// Chapters
	Helper::checkMenu(chapters_menu, core->mset.current_chapter_id);

	// Angles
	Helper::checkMenu(angles_menu, core->mset.current_angle_id);

	// Deinterlace menu
	Helper::checkMenu( deinterlace_menu, core->mset.current_deinterlacer);

	// Video size menu
	Helper::checkMenu( videosize_menu, pref->zoom );

	// Auto phase
	videofilter_menu->setItemChecked( VIDEO_FILTER_AUTOPHASE, core->mset.phase_filter );

	// Denoise
	videofilter_menu->setItemChecked( VIDEO_FILTER_DENOISE_SOFT, core->mset.current_denoiser == DENOISE_SOFT );
	videofilter_menu->setItemChecked( VIDEO_FILTER_DENOISE_NORMAL, core->mset.current_denoiser == DENOISE_NORMAL );

	// Deblock
	videofilter_menu->setItemChecked( VIDEO_FILTER_DEBLOCK, core->mset.deblock_filter );

	// Dering
	videofilter_menu->setItemChecked( VIDEO_FILTER_DERING, core->mset.dering_filter );

	// Add noise
	videofilter_menu->setItemChecked( VIDEO_FILTER_NOISE, core->mset.noise_filter );

	// Postprocessing
	videofilter_menu->setItemChecked( VIDEO_FILTER_POSTPROCESSING, core->mset.postprocessing_filter );


	// Fullscreen button
	fullscreenbutton->setOn(pref->fullscreen);

	// Mute button
	mutebutton->setOn(core->mset.mute);
	if (core->mset.mute) 
		mutebutton->setPixmap( Images2::icon("mute_small") );
	else
		mutebutton->setPixmap( Images2::icon("volume_small") );

	// Volume slider
	volumeslider->setValue( core->mset.volume );

	// Mute menu option
	audioMenu->setItemChecked( AUDIO_MUTE, core->mset.mute );

	// Karaoke menu option
	audiofilter_menu->setItemChecked( AUDIO_FILTER_KARAOKE, core->mset.karaoke_filter );

	// Extrastereo menu option
	audiofilter_menu->setItemChecked( AUDIO_FILTER_EXTRASTEREO, core->mset.extrastereo_filter );

	// Repeat menu option
	playMenu->setItemChecked( PLAY_REPEAT, pref->loop );

	// Time slider
	if (core->state==STOPPED) {
		//FIXME
		//timeslider->setValue( (int) core->mset.current_sec );
	}

	// Video equalizer
	videoMenu->setItemChecked( VIDEO_EQUALIZER, equalizer->isVisible() );

	// Playlist
	optionsMenu->setItemChecked( OPTIONS_PLAYLIST, playlist->isVisible() );

	// Frame counter
	optionsMenu->setItemChecked( OPTIONS_FRAMECOUNTER, pref->show_frame_counter );

	// Compact mode
	videoMenu->setItemChecked( VIDEO_COMPACT_MODE, pref->compact_mode );

	// Stay on top
	videoMenu->setItemChecked( VIDEO_STAY_ON_TOP, pref->stay_on_top );
}

void Gui::updateEqualizer() {
	// Equalizer
	equalizer->contrast->setValue( core->mset.contrast );
	equalizer->brightness->setValue( core->mset.brightness );
	equalizer->hue->setValue( core->mset.hue );
	equalizer->saturation->setValue( core->mset.saturation );
	equalizer->gamma->setValue( core->mset.gamma );
}


void Gui::updatePlaylistButton() {
	bool visible = playlist->isVisible();
	playlistbutton->setOn( visible );
	optionsMenu->setItemChecked( OPTIONS_PLAYLIST, visible );

#if DOCK_PLAYLIST
	/*
	Dock dock;
	int index, extraOffset;
	bool nl;
	if (getLocation(playlistdock, dock, index, nl, extraOffset)) {
		if ( (dock==Qt::DockTop) || (dock==Qt::DockBottom) ) {
			if (visible) {
				resize( size().width(), size().height() + playlistdock->size().height() );
			} else {
				resize( size().width(), size().height() - playlistdock->size().height() );
			}
		}
		else
		if ( (dock==Qt::DockLeft) || (dock==Qt::DockRight) ) {
			if (visible) {
				resize( size().width() + playlistdock->size().width(), size().height() );
			} else {
				resize( size().width() - playlistdock->size().width(), size().height() );
			}
		}
	}
	*/
#endif
}

void Gui::openRecent(int item) {
	qDebug("Gui::openRecent: %d", item);
	if ((item > -1) && (item < RECENTS_CLEAR)) { // 1000 = Clear item
		open( recents->item(item) );
	}
}

void Gui::open(QString file) {
	// If file is a playlist, open that playlist
	QString extension = QFileInfo(file).extension(FALSE).lower();
	if ( (extension=="m3u") || (extension=="m3u8") ) {
		playlist->load_m3u(file);
	} 
	else 
	if (QFileInfo(file).isDir()) {
		openDirectory(file);
	} 
	else {
		// Let the core to open it, autodetecting the file type
		core->open(file);
	}
}

void Gui::openURL() {
	qDebug("Gui::openURL");

	exitFullscreenIfNeeded();

    bool ok;
    QString s = QInputDialog::getText(
            "SMplayer", "URL:", QLineEdit::Normal,
            pref->last_url, &ok, this );

    if ( ok && !s.isEmpty() ) {

		//playlist->clear();
		//playlistdock->hide();

		openURL(s);
    } else {
        // user entered nothing or pressed Cancel
    }
}

void Gui::openURL(QString url) {
	if (!url.isEmpty()) {
		pref->last_url = url;
		core->openStream(url);
	}
}


void Gui::openFile() {
	qDebug("Gui::fileOpen");
	
	exitFullscreenIfNeeded();

    QString s = QFileDialog::getOpenFileName(pref->latest_dir, 
                       tr("Video") +" (*.avi *.mpg *.mpeg *.mkv *.wmv *.ogm *.vob *.flv *.mov *.ts *.rmvb);;" +
                       tr("Audio") +" (*.mp3 *.ogg *.wav *.wma);;" +
                       tr("Playlists") +" (*.m3u *.m3u8);;" +
                       tr("All files") +" (*.*)", +
                        this, 
                        "open file dialog", 
                        tr("Choose a file") );

    if ( !s.isEmpty() ) {
		openFile(s);
	}
}

void Gui::openFile(QString file) {
   if ( !file.isEmpty() ) {

		//playlist->clear();
		//playlistdock->hide();

		// If file is a playlist, open that playlist
		QString extension = QFileInfo(file).extension(FALSE).lower();
		if ( (extension=="m3u") || (extension=="m3u8") ) {
			playlist->load_m3u(file);
		} else {
			pref->latest_dir = QFileInfo(file).dirPath(TRUE);
			core->openFile(file);
		}
	}
}

void Gui::openDVD() {
	qDebug("Gui::openDVD");
	//core->openDVD(FALSE);

	if ( (pref->dvd_device.isEmpty()) || 
         (pref->cdrom_device.isEmpty()) )
	{
		QMessageBox::information( this, tr("SMPlayer - Information"),
			tr("The CDROM / DVD drives are not configured yet.\n"
			   "The configuration dialog will be shown now, "
               "so you can do it."), QMessageBox::Ok);
	
		showPreferencesDialog();
		pref_dialog->showSection( PreferencesDialog::Drives );
	} else {
		core->openDVD("dvd://1");
	}
}

void Gui::openDVDFromFolder() {
	qDebug("Gui::openDVDFromFolder");

	InputDVDDirectory *d = new InputDVDDirectory(this);
	d->setFolder( pref->last_dvd_directory );

	if (d->exec() == QDialog::Accepted) {
		qDebug("Gui::openDVDFromFolder: accepted");
		openDVDFromFolder( d->folder() );
	}

	delete d;
}

void Gui::openDVDFromFolder(QString directory) {
	//core->openDVD(TRUE, directory);
	pref->last_dvd_directory = directory;
	core->openDVD( "dvd://1:" + directory);
}

void Gui::openDirectory() {
	qDebug("Gui::openDirectory");

	QString s = QFileDialog::getExistingDirectory(
                    pref->latest_dir,
                    this,
                    "get existing directory",
                    tr("Choose a directory"),
                    TRUE );

	if (!s.isEmpty()) {
		openDirectory(s);
	}
}

void Gui::openDirectory(QString directory) {
	qDebug("Gui::openDirectory: '%s'", directory.utf8().data());

	if (Helper::directoryContainsDVD(directory)) {
		core->open(directory);
	} 
	else {
		QFileInfo fi(directory);
		if ( (fi.exists()) && (fi.isDir()) ) {
			playlist->clear();
			//playlist->addDirectory(directory);
			playlist->addDirectory( fi.absFilePath() );
			playlist->startPlay();
		} else {
			qDebug("Gui::openDirectory: directory is not valid");
		}
	}
}

void Gui::loadSub() {
	qDebug("Gui::loadSub");

	exitFullscreenIfNeeded();

    QString s = QFileDialog::getOpenFileName(
	    pref->latest_dir, 
        tr("Subtitles") +" (*.srt *.sub *.ssa *.ass *.idx"
                         " *.txt *.smi *.rt *.utf *.aqt);;" +
        tr("All files") +" (*.*)", 
        this, "open_file_dialog", tr("Choose a file") );

	if (!s.isEmpty()) core->loadSub(s);
}

void Gui::helpAbout() {
    QMessageBox::about(this, tr("About SMplayer"), 
		 "<b>SMPlayer</b> (c) 2006-2007 RVM &lt;rvm@escomposlinux.org&gt;<br><br>"
		 "<b>" + tr("Version: %1").arg(VERSION) + "</b><br><br>" +
         tr("Qt version: %1").arg(QT_VERSION_STR) + "<br><br>" 
		 "<i>" +
		 tr("This program is free software; you can redistribute it and/or modify "
	     "it under the terms of the GNU General Public License as published by "
	     "the Free Software Foundation; either version 2 of the License, or "
  	     "(at your option) any later version.") +
         "</i><br><br>" +
		 tr("Translators:") + "<br>"
         "<ul>"
         "<li>Henrikx <i>&lt;forum.gleitz.info&gt;</i> (" + tr("German") + ")</li>"
         "<li>Sweto &lt;peter.mendel@gmail.com&gt; (" +tr("Slovak") + ")</li>"
         "<li>Giancarlo Scola &lt;giancarlo@codexcoop.it&gt; (" +tr("Italian") + ")</li>"
         "<li>Olivier g &lt;1got@caramail.com&gt; (" +tr("French") + ")</li>"
         "<li>Tim Green &lt;iamtimgreen@gmail.com&gt; (" +tr("Simplified-Chinese") + ")</li>"
         "<li>Yurkovsky Andrey &lt;anyr@tut.by&gt; (" +tr("Russian") + ")</li>"
         "<li>Charles Barcza &lt;kbarcza@blackpanther.hu&gt; (" +tr("Hungarian") + ")</li>"
         "<li>" + tr("%1 and %2 (Polish)").arg("qla &lt;qla0@vp.pl&gt;").arg("Jarek &lt;ajep9691@wp.pl&gt;") +"</li>"
         "<li>Nardog &lt;nardog@e2umail.com&gt; (" +tr("Japanese") + ")</li>"
         "<li>Wesley S. &lt;wesley@ubuntu-nl.org&gt; (" +tr("Dutch") + ")</li>"
         "<li>Motsyo Gennadi &lt;drool@altlinux.ru&gt; (" +tr("Ukrainian") + ")</li>"
         "<li>Ventura &lt;ventura.barbeiro@terra.com.br&gt; (" +tr("Brazilian Portuguese") + ")</li>"
         "</ul>" +
		 tr("Logo designed by %1").arg("Charles Barcza &lt;kbarcza@blackpanther.hu&gt;") +
         "<br><br>" +
		 tr("Get updates at: %1")
         .arg("<a href=\"http://smplayer.sourceforge.net/\">http://smplayer.sourceforge.net/</a>")
		);
}

void Gui::helpAboutQt() {
	QMessageBox::aboutQt(this, tr("About Qt") );
}


void Gui::toggleFullscreen() {
	qDebug("Gui::toggleFullscreen");

	toggleFullscreen(!pref->fullscreen);
}

void Gui::toggleFullscreen(bool b) {
	qDebug("Gui::toggleFullscreen: %d", b);

	if (b==pref->fullscreen) {
		// Nothing to do
		return;
	}

	pref->fullscreen = b;

	// If using mplayer window
	if (pref->use_mplayer_window) {
		core->tellmp("vo_fullscreen " + QString::number(b) );
		updateWidgets();
		return;
	}

	if (!panel->isVisible()) return; // mplayer window is not used.

#if FULLSCREEN_WITH_MAINWINDOW
	if (pref->fullscreen) {
		/*
		setWFlags(Qt::WType_TopLevel | Qt::WStyle_Customize | 
                  Qt::WStyle_NoBorder | Qt::WStyle_StaysOnTop );
		*/
		#ifdef Q_OS_WIN
		win_pos = pos();
		win_size = size();
		hide();
		#endif
		if (!pref->compact_mode) {
			menuBar()->hide();
			statusBar()->hide();
			controlwidget->hide();
		}
		showFullScreen();
	} else {
		/* setWFlags(Qt::WType_TopLevel); */
		#ifdef Q_OS_WIN
		hide();
		#endif
		if (!pref->compact_mode) {
			menuBar()->show();
			statusBar()->show();
			controlwidget->show();
		}
		showNormal();
		#ifdef Q_OS_WIN
		move( win_pos );
		resize( win_size );
		#endif
	}
#else
    if (pref->fullscreen) {
		#if QT_VERSION < 0x040000
		fullscreen_widget->resize( desktop_size() );
		fullscreen_widget->showFullScreen();
		mplayerwindow->reparent( fullscreen_widget, 0, QPoint(0,0), TRUE );
		fullscreen_widget_layout->addWidget( mplayerwindow, 0, 0 );
		#else
		// This way produces a nice switching to fullscreen, but
		// unfortunately it doesn't work well with Qt 3
		fullscreen_widget->resize( desktop_size() );
		mplayerwindow->reparent( fullscreen_widget, 0, QPoint(0,0), TRUE );
		fullscreen_widget_layout->addWidget( mplayerwindow, 0, 0 );
		fullscreen_widget->showFullScreen();
		#endif
		mplayerwindow->setFocus(); // Needed for windows
    } else {
		// Try to do the switching smoother.
		//fullscreen_widget->hide();
		mplayerwindow->hide();
		fullscreen_widget->hide();
		mplayerwindow->resize( panel->size() );
		mplayerwindow->reparent(panel, 0, QPoint(0,0), FALSE);
		panelLayout->addWidget( mplayerwindow, 0, 0 );
		//fullscreen_widget->hide();
		mplayerwindow->show();
    }
#endif
	updateWidgets();
}


void Gui::displayTime(double sec) {
	if (floor(sec)==last_second) return; // Update only once per second
	last_second = (int) floor(sec);

	time_display->setText( Helper::formatTime( (int) sec ) + " / " +
                           Helper::formatTime( (int) core->mdat.duration ) );

	//qDebug( " duration: %f, current_sec: %f", core->mdat.duration, core->mset.current_sec);

    //Update slider
	if ( (core->mdat.duration > 1) && (core->mset.current_sec > 1) && 
         (core->mdat.duration > core->mset.current_sec) ) 
	{
	    int perc = ( (int) core->mset.current_sec * 100) / (int) core->mdat.duration;
	    timeslider->setPos(perc);
	}
}

void Gui::displayFrame(int frame) {
	if (frame_display->isVisible()) {
		frame_display->setText( " " + QString::number(frame) + " " );
	}
}

void Gui::toggleFrameCounter() {
	toggleFrameCounter( !pref->show_frame_counter );
}

void Gui::toggleFrameCounter(bool b) {
	frame_display->setShown(b);
	pref->show_frame_counter = b;
	updateWidgets();
}


void Gui::keyPressEvent( QKeyEvent *e ) {
	qDebug("Gui::keyPressEvent: '%s'", e->text().utf8().data());

	QString f = kl->function(e);
	qDebug("Gui::processKeyEvent: key function: '%s'", f.utf8().data() );

	if (f.isEmpty()) {
		e->ignore();
		return;
	} else {
		e->accept();
		processFunction(f);
	}
}

void Gui::processFunction(QString f) {
	qDebug("Gui::processFunction: '%s'", f.utf8().data());

	if (f=="PLAYLIST_NEXT") {
		playlist->playNext();
	}
	else
	if (f=="PLAYLIST_PREV") {
		playlist->playPrev();
	}
	else
	if (f=="EXIT_FULLSCREEN") {
        if (pref->fullscreen) {
			toggleFullscreen();
		}
    }
    else
	if (f=="FULLSCREEN") {
		toggleFullscreen();
	}
	else
	if (f=="VIDEO_EQUALIZER") {
		showEqualizer();
	}
	else
	if (f=="COMPACT_MODE") {
		toggleCompactMode();
	}
	else
	if (f=="OPEN_FILE") {
		openFile();
	}
	else
	if (f=="OPEN_URL") {
		openURL();
	}
	else
	if (f=="EXIT") {
		close();
	}
	else
	if (f=="INFO") {
		showInfo();
	}
	else
	if (f=="PREFERENCES") {
		showPreferencesDialog();
	}
	else
	if (f=="MPLAYER_LOG") {
		showMplayerLog();
	}
	else
	if (f=="SMPLAYER_LOG") {
		showLog();
	}
	else
	if (f=="PLAYLIST") {
		showPlaylist();
	}
	else {
		//core->processKeyEvent(e);
		core->processFunction(f);
	}
}

void Gui::leftClickFunction() {
	qDebug("Gui::leftClickFunction");
	if (!pref->left_click_function.isEmpty()) {
		processFunction(pref->left_click_function);
	}
}

void Gui::doubleClickFunction() {
	qDebug("Gui::doubleClickFunction");
	if (!pref->double_click_function.isEmpty()) {
		processFunction(pref->double_click_function);
	}
}

void Gui::dragEnterEvent( QDragEnterEvent *e ) {
	qDebug("Gui::dragEnterEvent");
	//qDebug(" text: %d", QTextDrag::canDecode(e));
	#if QT_VERSION < 0x040000
	e->accept( (QUriDrag::canDecode(e) || QTextDrag::canDecode(e)) );
	#else
	e->accept( QUriDrag::canDecode(e) );
	#endif
}

void Gui::dropEvent( QDropEvent *e ) {
	qDebug("Gui::dropEvent");

	QStringList files;

	if (QUriDrag::canDecode(e)) {
		QStrList l;
		QUriDrag::decode(e, l);

		QString s;
        for ( unsigned int i= 0; i < l.count(); ++i ) {
			s = l.at(i);
			qDebug(" * '%s'", s.utf8().data() );
			QUrl u(s);
			qDebug(" * protocol: '%s'", u.protocol().utf8().data());
			qDebug(" * path: '%s'", u.path().utf8().data());
			//qDebug(" filename:='%s'", u.fileName().utf8().data());
			if (u.protocol()=="file") {
				s = u.path();
			}
			files.append( s );
		}
	}

	#if QT_VERSION < 0x040000
	else
	if (QTextDrag::canDecode(e)) {
		QString s;
		if (QTextDrag::decode(e, s)) {
			qDebug(" * '%s'", s.utf8().data() );
			files.append( s );
		}
	}
	#endif

	qDebug( " count: %d", files.count());
	if (files.count() > 0) {
		if (files.count() == 1) {
			QFileInfo fi( files[0] );
			if (fi.isDir()) {
				openDirectory( files[0] );
			} else {
				//openFile( files[0] );
				open( files[0] );
			}
		} else {
			playlist->clear();
			playlist->addFiles(files);
			//openFile( files[0] );
			playlist->startPlay();
		}
	}
}

void Gui::showPopupMenu( QPoint p ) {
	qDebug("Gui::showPopupMenu");

	popup->move( p );
	popup->show();
}

void Gui::mouseReleaseEvent( QMouseEvent * e ) {
	qDebug("Gui::mouseReleaseEvent");

	if (e->button() == Qt::LeftButton) {
		e->accept();
		emit leftClicked();
	}
	/*
	else
	if (e->button() == Qt::RightButton) {
		showPopupMenu( e->globalPos() );
    }
	*/
	else 
		e->ignore();
}

void Gui::mouseDoubleClickEvent( QMouseEvent * e ) {
	e->accept();
	emit doubleClicked();
}

void Gui::wheelEvent( QWheelEvent * e ) {
	qDebug("Gui::wheelEvent: delta: %d", e->delta());
	e->accept();

	if (e->delta() >= 0) 
		emit wheelUp();
	else
		emit wheelDown();
}


// Called when a video has started to play
void Gui::enterFullscreen() {
	if ( (pref->start_in_fullscreen) && (!pref->fullscreen) ) {
		toggleFullscreen(TRUE);
	}
}

// Called when the playlist has stopped
void Gui::exitFullscreen() {
    if (pref->fullscreen) {
		toggleFullscreen(FALSE);
	}
}

void Gui::displayState(int state) {
	qDebug("Gui::displayState: %d", state);
	switch (state) {
		case PLAYING:	statusBar()->message( tr("Playing %1").arg(core->mdat.filename), 2000); break;
		case PAUSED:	statusBar()->message( tr("Pause") ); break;
		case STOPPED:	statusBar()->message( tr("Stop") , 2000); break;
	}
}

void Gui::displayMessage(QString message) {
	statusBar()->message(message, 2000);
}


void Gui::resizeWindow(int w, int h) {
	qDebug("Gui::resizeWindow: %d, %d", w, h);

	#if FULLSCREEN_WITH_MAINWINDOW
	// If fullscreen, don't resize!
	if (pref->fullscreen) return;
	#endif

	if ( (pref->resize_method==RESIZE_NEVER) && (panel->isVisible()) ) {
		return;
	}

	if (!panel->isVisible()) {
		//hide();
		panel->show();
		QPoint p = pos();
		adjustSize();
		move(p);
		//show();
	}

	if (pref->zoom != 100) {
		double zoom = (double) pref->zoom/100;
		w = w * zoom;
		h = h * zoom;
	}

	int width = size().width() - panel->size().width();
	int height = size().height() - panel->size().height();

	width += w;
	height += h;

	resize(width,height);

	qDebug("width: %d, height: %d", width, height);
	qDebug("mplayerwindow->size: %d, %d", 
           mplayerwindow->size().width(),  
           mplayerwindow->size().height() );

	mplayerwindow->setFocus(); // Needed?
}

void Gui::hidePanel() {
	if (panel->isVisible()) {
		//resizeWindow( size().width(), 0 );
		resize( size().width(), size().height() - panel->size().height() );
		panel->hide();
	}
}

void Gui::displayGotoTime(int t) {
	int jump_time = (int)core->mdat.duration * t / 100;
	//QString s = tr("Jump to %1").arg( Helper::formatTime(jump_time) );
	QString s = QString("Jump to %1").arg( Helper::formatTime(jump_time) );
	statusBar()->message( s, 1000 );

	if (pref->fullscreen) {
		core->tellmp("osd_show_text \"" + s + "\" 3000 1");
	}
}

void Gui::toggleCompactMode() {
	toggleCompactMode( !pref->compact_mode );
}

void Gui::toggleCompactMode(bool b) {
	menuBar()->setHidden(b);
	statusBar()->setHidden(b);
	controlwidget->setHidden(b);

	pref->compact_mode = b;
	updateWidgets();
}

void Gui::toggleStayOnTop() {
	toggleStayOnTop( !pref->stay_on_top );
}

void Gui::toggleStayOnTop(bool b) {
	bool visible = isVisible();

	QPoint old_pos = pos();

	if (b) {
		#if QT_VERSION >= 0x040000
		setWindowFlags(Qt::WStyle_StaysOnTop);
		#else
		reparent(0, Qt::WStyle_StaysOnTop, pos() );
		#endif
	}
	else {
		#if QT_VERSION >= 0x040000
		setWindowFlags(0);
		#else
		reparent(0, 0, pos() );
		#endif
	}

	move(old_pos);

	if (visible) {
		show();
	}

	pref->stay_on_top = b;

	updateWidgets();
}


// Called when a new window (equalizer, preferences..) is opened.
void Gui::exitFullscreenIfNeeded() {
	#if !FULLSCREEN_WITH_MAINWINDOW
	if (pref->fullscreen) {
		toggleFullscreen(FALSE);
	}
	#endif
}

void Gui::checkMousePos(QPoint p) {
#if FULLSCREEN_WITH_MAINWINDOW
	//qDebug("Gui::checkMousePos: %d, %d", p.x(), p.y());

	if (!pref->fullscreen) return;

	// Show controlwidget if mouse is near the botton
	#define MARGIN 30
	if (p.y() > mplayerwindow->height() - MARGIN) {
		qDebug("Gui::checkMousePos: %d, %d", p.x(), p.y());
		if (!controlwidget->isVisible()) controlwidget->show();
	} else {
		if (controlwidget->isVisible()) controlwidget->hide();
	}

	// Show menubar if mouse is near the top
	if (p.y() < MARGIN) {
		qDebug("Gui::checkMousePos: %d, %d", p.x(), p.y());
		if (!menuBar()->isVisible()) menuBar()->show();
	} else {
		if (menuBar()->isVisible()) menuBar()->hide();
	}
#endif
}

